import { Component } from "./Component";
import { XML_CHILDREN } from "./constants";
import { Button } from "./Button";
import { PrintCell } from "./PrintCell";
import { Checkbox } from "./Checkbox";
import { DatePicker } from "./DatePicker";
import { Label } from "./Label";
import { PrintForm } from "./PrintForm";
import { Grid } from "./Grid";
import { Image } from "./Image";
import { Page } from "./Page";
import { SignatureCapture } from "./SignatureCapture";
import { TextArea } from "./TextArea";
import { TextField } from "./TextField";
import { TimePicker } from "./TimePicker";
import { ValuePicker } from "./ValuePicker";
import { PrintUtil } from "./PrintUtil";
import { XML_BUTTON, XML_CELL, XML_CHECKBOX, XML_DATE_PICKER, XML_FORM, XML_GRID, XML_IMAGE, XML_LABEL, XML_PAGE, XML_SIGNATURE_CAPTURE, XML_TEXT_AREA, XML_TEXT_FIELD, XML_TIME_PICKER, XML_VALUE_PICKER } from "./constants";
export class Container extends Component {
    constructor(parentContainer, node, overrideContainerWidth) {
        super(parentContainer, node);
        this._children = new Array();
        if (this.nodeChildDict[XML_CHILDREN]) {
            PrintUtil.forEachChildNode(this.nodeChildDict[XML_CHILDREN], (n) => {
                let c = ComponentFactory.fromNode(n, this);
                if (c) {
                    this._children.push(c);
                }
            });
        }
    }
    get children() {
        return this._children;
    }
    ;
    get containerWidth() {
        if (!this._containerWidth) {
            this.calcAndAssignContainerWidth(this.parentContainer);
        }
        return this._containerWidth;
    }
    assignChildren(c) {
        this._children = c;
    }
    assignContainerWidth(width) {
        this._containerWidth = width;
    }
    assignParentWidth(parentWidth) {
        this.assignContainerWidth(this.layout.sumOfWidths(parentWidth));
    }
    calcAndAssignContainerWidth(parentContainer) {
        // Overriden by Form and Cell for altered behavior
        let parentWidth = parentContainer.containerWidth;
        if (this.layout == null) {
            throw Error("Bogus");
        }
        this.assignContainerWidth(this.layout.sumOfWidths(parentWidth));
    }
    initContainerWith(overrideLayout, overideChildren) {
        this.initComponentWith(overrideLayout);
        if (overideChildren) {
            this.assignChildren(overideChildren);
        }
    }
}
class ComponentFactory {
    static fromNode(node, parentContainer) {
        var answer = null;
        switch (node.nodeName) {
            case XML_BUTTON:
                answer = new Button(parentContainer, node);
                break;
            case XML_CHECKBOX:
                answer = new Checkbox(parentContainer, node);
                break;
            case XML_DATE_PICKER:
                answer = new DatePicker(parentContainer, node);
                break;
            case XML_IMAGE:
                answer = new Image(parentContainer, node);
                break;
            case XML_LABEL:
                answer = new Label(parentContainer, node);
                break;
            case XML_SIGNATURE_CAPTURE:
                answer = new SignatureCapture(parentContainer, node);
                break;
            case XML_TEXT_AREA:
                answer = new TextArea(parentContainer, node);
                break;
            case XML_TEXT_FIELD:
                answer = new TextField(parentContainer, node);
                break;
            case XML_TIME_PICKER:
                answer = new TimePicker(parentContainer, node);
                break;
            case XML_VALUE_PICKER:
                answer = new ValuePicker(parentContainer, node);
                break;
            case XML_CELL:
                answer = new PrintCell(parentContainer, node);
                break;
            case XML_FORM:
                answer = new PrintForm(node);
                break;
            case XML_GRID:
                answer = new Grid(parentContainer, node);
                break;
            case XML_PAGE:
                answer = new Page(parentContainer, node);
                break;
        }
        return answer;
    }
}
