import { Container } from "./Container";
import { Point } from "./Point";
import { GridLine } from "./GridLine";
export class Grid extends Container {
    constructor(parentContainer, node) {
        super(parentContainer, node);
    }
    get gridLines() {
        if (!this._gridLines) {
            this._gridLines = new Array();
            let hasHLines = false;
            let hasVLines = false;
            let cols = this.layout.widths.length;
            let rows = this.layout.heights.length;
            let vLines = new Array(rows * cols + rows); // Distinct vertical lines
            let hLines = new Array(rows * cols + cols); // Distinct horizointal lines
            let colStart = 0;
            for (let c = 0; c < cols; c++) {
                let rowStart = 0;
                let colWidth = this.layout.widths[c].resolveWithFill(this.parentContainer.containerWidth);
                for (let r = 0; r < rows; r++) {
                    let rowHeight = this.layout.heights[r].resolveWithFill(this.parentContainer.containerWidth);
                    // Vertical lines
                    let lineWidth = 0;
                    if (c == 0) {
                        // Left most vertical
                        lineWidth = this.cellChildren[r][c].borderWidths.left;
                    }
                    else {
                        // Middle verticals
                        lineWidth = Math.max(this.cellChildren[r][c - 1].borderWidths.right, this.cellChildren[r][c].borderWidths.left);
                    }
                    if (lineWidth) {
                        vLines[c * rows + r] = new GridLine(new Point(colStart, rowStart), new Point(colStart, rowStart + rowHeight), lineWidth);
                        hasVLines = true;
                    }
                    if (c == cols - 1) {
                        // Right most vertical
                        lineWidth = this.cellChildren[r][c].borderWidths.right;
                        if (lineWidth) {
                            // Place these appropriately so the combining logic can work.
                            vLines[cols * rows + r] = new GridLine(new Point(colStart + colWidth, rowStart), new Point(colStart + colWidth, rowStart + rowHeight), lineWidth);
                            hasVLines = true;
                        }
                    }
                    // Horizontal lines
                    lineWidth = 0;
                    if (r == 0) {
                        // Top most horizontal
                        lineWidth = this.cellChildren[r][c].borderWidths.top;
                    }
                    else {
                        // Middle horizontals
                        lineWidth = Math.max(this.cellChildren[r - 1][c].borderWidths.bottom, this.cellChildren[r][c].borderWidths.top);
                    }
                    if (lineWidth) {
                        hLines[r * cols + c] = new GridLine(new Point(colStart, rowStart), new Point(colStart + colWidth, rowStart), lineWidth);
                        hasHLines = true;
                    }
                    if (r == rows - 1) {
                        // Bottom most horizontal
                        lineWidth = this.cellChildren[r][c].borderWidths.bottom;
                        if (lineWidth) {
                            // Place these appropriately so the combining logic can work.
                            hLines[cols * rows + c] = new GridLine(new Point(colStart, rowStart + rowHeight), new Point(colStart + colWidth, rowStart + rowHeight), lineWidth);
                            hasHLines = true;
                        }
                    }
                    rowStart += rowHeight;
                }
                colStart += colWidth;
            }
            // Combine adjacent lines
            let lastX = NaN;
            let lastEndY = NaN;
            let lastLineWidth = NaN;
            let lastPush = this._gridLines.length - 1;
            if (hasVLines) {
                for (let gl of vLines) {
                    if (gl) {
                        if (isNaN(lastEndY)) {
                            lastEndY = gl.end.y;
                            lastX = gl.end.x;
                            lastLineWidth = gl.lineWidth;
                            this._gridLines.push(gl);
                            lastPush++;
                        }
                        else if (gl.start.y == lastEndY && (gl.start.x == lastX) && (gl.lineWidth == lastLineWidth)) {
                            // This line and the previous can be combined
                            lastEndY = gl.end.y;
                            this._gridLines[lastPush].end = gl.end;
                        }
                        else {
                            this._gridLines.push(gl);
                            lastPush++;
                            lastEndY = gl.end.y;
                            lastX = gl.end.x;
                            lastLineWidth = gl.lineWidth;
                        }
                    }
                }
            }
            if (hasHLines) {
                let lastEndX = NaN;
                let lastY = NaN;
                lastLineWidth = NaN;
                lastPush = this._gridLines.length - 1;
                for (let gl of hLines) {
                    if (gl) {
                        if (isNaN(lastEndX)) {
                            lastEndX = gl.end.x;
                            lastY = gl.end.y;
                            lastLineWidth = gl.lineWidth;
                            this._gridLines.push(gl);
                            lastPush++;
                        }
                        else if (gl.start.x == lastEndX && (gl.start.y == lastY) && (gl.lineWidth == lastLineWidth)) {
                            // This line and the previous can be combined
                            lastEndX = gl.end.x;
                            this._gridLines[lastPush].end = gl.end;
                        }
                        else {
                            this._gridLines.push(gl);
                            lastPush++;
                            lastEndX = gl.end.x;
                            lastY = gl.end.y;
                            lastLineWidth = gl.lineWidth;
                        }
                    }
                }
            }
        }
        return this._gridLines;
    }
    initCells() {
        // Structure the cells so that they can be retrieved
        this._cellChildren = new Array(this.layout.heights.length);
        for (var i = 0; i < this.layout.heights.length; i++) {
            this._cellChildren[i] = new Array(this.layout.widths.length);
        }
        this.children.map((c) => {
            this._cellChildren[c.layout.row][c.layout.column] = c;
        });
    }
    get cellChildren() {
        if (this._cellChildren == null) {
            this.initCells();
        }
        return this._cellChildren;
    }
    initGridWith(overrideLayout, overrideChildren) {
        super.initContainerWith(overrideLayout, overrideChildren);
        this.initCells();
    }
}
