import { Spec } from "./Spec";
import { RichNumUsage } from "./constants";
import { RichNum } from "./RichNum";
import { PrintUtil } from "./PrintUtil";
import { XML_COLUMN, XML_FILL_PARENT, XML_ORIGIN, XML_ROW, XML_SIZE, XML_UOM } from "./constants";
export class Layout extends Spec {
    constructor(node, x, y, width, height, row, col) {
        super(node);
        if (node) {
            PrintUtil.ifChild(this.nodeChildDict[XML_UOM], (n) => {
                this._uom = PrintUtil.singleChildText(n);
            });
            PrintUtil.ifChild(this.nodeChildDict[XML_COLUMN], (n) => {
                this._column = PrintUtil.singleChildInt(n);
            });
            PrintUtil.ifChild(this.nodeChildDict[XML_ROW], (n) => {
                this._row = PrintUtil.singleChildInt(n);
            });
            PrintUtil.ifChild(this.nodeChildDict[XML_SIZE], (n) => {
                this._heights = PrintUtil.arrayOfRichNums(n, "Height");
                this._widths = PrintUtil.arrayOfRichNums(n, "Width");
            });
            PrintUtil.ifChild(this.nodeChildDict[XML_FILL_PARENT], (n) => {
                this._heights = [new RichNum(NaN, RichNumUsage.FillParent)];
                this._widths = [new RichNum(NaN, RichNumUsage.FillParent)];
            });
            PrintUtil.ifChild(this.nodeChildDict[XML_ORIGIN], (n) => {
                PrintUtil.forEachChildNode(n, (n2) => {
                    switch (n2.nodeName) {
                        case "X":
                            this._x = PrintUtil.singleChildFloat(n2);
                            break;
                        case "Y":
                            this._y = PrintUtil.singleChildFloat(n2);
                            break;
                    }
                });
            });
        }
        else {
            this._x = x;
            this._y = y;
            this._widths = [new RichNum(width)];
            this._heights = [new RichNum(height)];
            this._column = col;
            this._row = row;
        }
    }
    get uom() {
        return this._uom;
    }
    get heights() {
        return this._heights;
    }
    get widths() {
        return this._widths;
    }
    get x() {
        return this._x;
    }
    get y() {
        return this._y;
    }
    get column() {
        return this._column;
    }
    get row() {
        return this._row;
    }
    singleHeightNum() {
        let rn = this.singleHeightRichNum();
        if (!rn.isNumber) {
            throw Error("Expecting number on height layout");
        }
        return rn.value;
    }
    singleHeightRichNum() {
        if (!this.heights) {
            throw Error("No height values on Layout");
        }
        else if (this.heights.length != 1) {
            throw Error("Expecting exactly 1 height, but found: " + this.heights.length);
        }
        else {
            return this.heights[0];
        }
    }
    singleWidthNum() {
        let rn = this.singleWidthRichNum();
        if (!rn.isNumber) {
            throw Error("Expecting number on width layout");
        }
        return rn.value;
    }
    singleWidthRichNum() {
        if (!this.widths) {
            throw Error("No width values on Layout");
        }
        else if (this.widths.length != 1) {
            throw Error("Expecting exactly 1 width, but found: " + this.widths.length);
        }
        else {
            return this.widths[0];
        }
    }
    sumOfHeights() {
        let answer = 0.0;
        if (!this.heights) {
            throw Error("No height values on Layout");
        }
        this.heights.map((rn) => {
            if (!rn) {
                throw Error("Unsupported ricn num usage on layout.heights");
            }
            if (!rn.isNumber) {
                throw Error("Expecting number on layout.heights");
            }
            answer += rn.value;
        });
        return answer;
    }
    sumOfWidths(parentSize) {
        let answer = 0.0;
        if (!this.widths) {
            throw Error("No width values on Layout");
        }
        this.widths.map((rn) => {
            if (!rn) {
                throw Error("Unsupported ricn num usage on layout.widths");
            }
            if (rn.isFillParent) {
                if (this.widths.length != 1) {
                    throw Error("More than one value being summed and FillParentWidth used: layout.widths");
                }
                answer = parentSize;
            }
            else if (rn.isPercentOfParent) {
                answer += parentSize * rn.valueOfPercent;
            }
            else if (rn.isNumber) {
                answer += rn.value;
            }
            else {
                throw Error("Unknown RichNum usage on layout.widths");
            }
        });
        return answer;
    }
}
