import { Container } from "./Container";
import { XML_BORDER_WIDTHS } from "./constants";
import { Color } from "./Color";
import { Layout } from "./Layout";
import { Edges } from "./Edges";
import { Grid } from "./Grid";
import { PrintUtil } from "./PrintUtil";
import { XML_BORDER_COLOR } from "./constants";
export class PrintCell extends Container {
    constructor(parentContainer, node) {
        super(parentContainer, node);
        this._grid = parentContainer;
        PrintUtil.ifChild(this.nodeChildDict[XML_BORDER_COLOR], (n) => {
            this._borderColor = new Color(n);
        });
        PrintUtil.ifChild(this.nodeChildDict[XML_BORDER_WIDTHS], (n) => {
            this._borderWidths = new Edges(n);
        });
    }
    get borderColor() {
        return this._borderColor;
    }
    get borderWidths() {
        return this._borderWidths;
    }
    get cellHeight() {
        return this._grid.layout.heights[this.layout.row].value;
    }
    get cellWidth() {
        return this._grid.layout.widths[this.layout.column].resolveWithFill(this._grid.containerWidth);
    }
    get componentChildren() {
        let answer = [];
        this.children.map((c) => {
            if (!(c instanceof Grid)) {
                answer.push(c);
            }
        });
        return answer;
    }
    get gridChildren() {
        let answer = [];
        this.children.map((c) => {
            if (c instanceof Grid) {
                answer.push(c);
            }
        });
        return answer;
    }
    initCellWith(overrideLayout, overrideChildren) {
        let ol = overrideLayout;
        if (!this.layout && !overrideLayout) {
            ol = new Layout(null, NaN, NaN, NaN, NaN, 0, 0);
        }
        this.initContainerWith(ol, overrideChildren);
        if (!this.borderWidths) {
            this._borderWidths = new Edges();
            this._borderWidths.initEdgesWith(0, 0, 0, 0);
        }
    }
    calcAndAssignContainerWidth(parentContainer) {
        if (parentContainer.layout == null) {
            throw Error("bogus");
        }
        let cw = parentContainer.layout.widths[this.layout.column].resolveWithFill(parentContainer.containerWidth);
        this.assignContainerWidth(cw);
    }
}
