import { RichNum } from "./RichNum";
import { TextAlignment } from "./TextAlignment";
import { Color } from "./Color";
import { Log } from "../util";
import { TextAlignmentUsage } from "./constants";
import { RichNumUsage, XML_BOLD, XML_ITALIC, XML_NUMBER_OF_LINES, XML_TEXT_ALIGNMENT, XML_TEXT_COLOR, XML_UNDERLINE } from "./constants";
export class PrintUtil {
    static arrayOfRichNums(node, name) {
        let answer = [];
        PrintUtil.forEachChildNode(node, (n) => {
            if (n.nodeName == name) {
                answer.push(PrintUtil.singleChildRichNum(n));
            }
        });
        return answer;
    }
    static enumValue(node, e) {
        let answer = null;
        let sv = PrintUtil.singleChildText(node);
        let nv;
        if (sv) {
            nv = e[sv];
        }
        return nv;
    }
    static forEachChildNode(node, f) {
        for (let i = 0; i < node.childNodes.length; i++) {
            f(node.childNodes[i]);
        }
    }
    static ifChild(node, f) {
        if (node) {
            f(node);
        }
    }
    static importTextAttributes(nodeChildDict, textAttributes) {
        PrintUtil.ifChild(nodeChildDict[XML_BOLD], (n) => {
            textAttributes.bold = PrintUtil.singleChildBoolean(n);
        });
        PrintUtil.ifChild(nodeChildDict[XML_ITALIC], (n) => {
            textAttributes.italic = PrintUtil.singleChildBoolean(n);
        });
        PrintUtil.ifChild(nodeChildDict[XML_UNDERLINE], (n) => {
            textAttributes.underline = PrintUtil.singleChildBoolean(n);
        });
        PrintUtil.ifChild(nodeChildDict[XML_TEXT_ALIGNMENT], (n) => {
            textAttributes.textAlignment = new TextAlignment(PrintUtil.singleChildTextAlignmentUsage(n));
        });
        PrintUtil.ifChild(nodeChildDict[XML_TEXT_COLOR], (n) => {
            textAttributes.textColor = new Color(n);
        });
        PrintUtil.ifChild(nodeChildDict[XML_NUMBER_OF_LINES], (n) => {
            textAttributes.numberOfLines = PrintUtil.singleChildInt(n);
        });
    }
    static singleChildBoolean(node) {
        let text = PrintUtil.singleChildText(node);
        if (text) {
            return text.toLocaleLowerCase() == "true";
        }
        else {
            return false;
        }
    }
    static singleChildInt(node) {
        var answer;
        if (node.childNodes.length != 1) {
            Log.error("XML error with " + node.nodeName + ".  Expected exactly one child node.");
        }
        else if (node.childNodes[0].nodeName != "#text") {
            Log.error("XML error with " + node.nodeName + ".  Expected numeric node.");
        }
        else {
            answer = parseInt(node.childNodes[0].textContent);
        }
        return answer;
    }
    static singleChildFloat(node) {
        var answer;
        if (node.childNodes.length != 1) {
            Log.error("XML error with " + node.nodeName + ".  Expected exactly one child node.");
        }
        else if (node.childNodes[0].nodeName != "#text") {
            Log.error("XML error with " + node.nodeName + ".  Expected numeric node.");
        }
        else {
            answer = parseFloat(node.childNodes[0].textContent);
        }
        return answer;
    }
    static singleChildRichNum(node) {
        // Either there is a FillParent entry with surrounding white space, or a single text entry
        var answer;
        for (let i = 0; i < node.childNodes.length; i++) {
            if (node.childNodes[i].nodeName == RichNumUsage[RichNumUsage.PercentOfParent].toString()) {
                let v = this.singleChildFloat(node.childNodes[i]);
                answer = new RichNum(v, RichNumUsage.PercentOfParent);
                break;
            }
            else if (node.childNodes[i].nodeName == RichNumUsage[RichNumUsage.FillParent].toString()) {
                answer = new RichNum(NaN, RichNumUsage.FillParent);
                break;
            }
            else if (node.childNodes[i].nodeName == "#text") {
                let v = parseFloat(node.childNodes[i].textContent.trim());
                if (!isNaN(v)) {
                    answer = new RichNum(v);
                }
                // Don't break, keep looking for FillParent
            }
        }
        return answer;
    }
    static singleChildText(node) {
        if (node.childNodes.length != 1) {
            var text = "ExpectedExactlyOneNode";
            Log.error("XML error with " + node.nodeName + ".  Expected exactly one child node.");
        }
        else if (node.childNodes[0].nodeName != "#text") {
            text = "ExpectedNodeText";
            Log.error("XML error with " + node.nodeName + ".  Expected text node.");
        }
        else {
            text = node.childNodes[0].textContent;
        }
        return text;
    }
    static singleChildTextAlignmentUsage(node) {
        // Either there is a FillParent entry with surrounding white space, or a single text entry
        var answer = TextAlignmentUsage.Left;
        if (node.childNodes.length != 1) {
            Log.error("XML error with " + node.nodeName + ".  Expected exactly one child node.");
        }
        else if (node.childNodes[0].nodeName != "#text") {
            Log.error("XML error with " + node.nodeName + ".  Expected text node.");
        }
        else {
            const a = node.childNodes[0].textContent;
            if (a == TextAlignmentUsage[TextAlignmentUsage.Center].toString()) {
                answer = TextAlignmentUsage.Center;
            }
            else if (a == TextAlignmentUsage[TextAlignmentUsage.Left].toString()) {
                answer = TextAlignmentUsage.Left;
            }
            else if (a == TextAlignmentUsage[TextAlignmentUsage.Right].toString()) {
                answer = TextAlignmentUsage.Right;
            }
            else {
                Log.error("XML error with " + node.nodeName + ".  Unknown TextAlignment: " + a);
            }
        }
        return answer;
    }
}
