import { Dialog } from "./Dialog";
import { Column } from "./Column";
import { Redirection } from "./Redirection";
import { Menu } from "./Menu";
import { RecordSet } from "./RecordSet";
import { QueryMarkerOption, QueryScroller } from "./QueryScroller";
import { QueryDirection } from "./types";
import { PositionalQueryAbilityType } from "./types";
/**
 * Dialog Subtype that represents a 'Query Dialog'.
 * A 'Query' represents and is backed by a list of Records and a single Record definition.
 * See {@link Record} and {@link RecordDef}.
 */
export declare class QueryDialog extends Dialog {
    private _scroller;
    private _defaultActionId;
    positionalQueryAbility: PositionalQueryAbilityType;
    supportsColumnStatistics: boolean;
    supportsPositionalQueries: boolean;
    readonly defaultActionId: string;
    initScroller(pageSize: number, firstObjectId?: string, markerOptions?: QueryMarkerOption[]): void;
    isBinary(column: Column): boolean;
    performMenuActionWithId(actionId: string, targets: string[]): Promise<{
        actionId: string;
    } | Redirection>;
    /**
     * Perform this action associated with the given Menu on this dialog.
     * The targets array is expected to be an array of object ids.
     * @param {Menu} menu
     * @param {Array<string>} targets
     * @returns {Promise<{actionId: string} | Redirection>}
     */
    performMenuAction(menu: Menu, targets: string[]): Promise<{
        actionId: string;
    } | Redirection>;
    /**
     * Perform a query
     *
     * @param {number} maxRows
     * @param {QueryDirection} direction
     * @param {string} fromObjectId
     * @returns {Promise<RecordSet>}
     */
    query(maxRows: number, direction: QueryDirection, fromObjectId: string): Promise<RecordSet>;
    /**
     * Get the associated QueryScroller
     * @returns {QueryScroller}
     */
    readonly scroller: QueryScroller;
    /**
     * Creates a new QueryScroller with default buffer size of 50
     * @returns {QueryScroller}
     */
    private defaultScroller();
}
